// Generated by CoffeeScript 1.10.0

/*
description: test jssgf.js
author: ICHIKAWA, Yuji
copyright: 2014 (C) ICHIKAWA, Yuji (New 3 Rs)
 */
var assert, parser;

assert = require('assert');

parser = require('./jssgf').parser;

describe('parser', function() {
  describe('bug of regular expression in V8', function() {
    it('bracket with linebreak', function() {
      var bracket, match;
      bracket = '[\n]';
      match = bracket.match(/\[[^\]]*\]/);
      return assert.equal(match[0], bracket);
    });
    return it('brackets with linebreak', function() {
      var a;
      a = '[][][][][][][][][][][][][][][][][][][][][][][][][][][][][\n])'.match(/(\[[^\]]*\])+\)$/);
      return assert.equal(typeof a, 'object');
    });
  });
  describe('collection', function() {
    it('should return the smallest collection', function() {
      return assert.deepEqual(parser.fastParse('(;)'), [
        {
          _children: []
        }
      ]);
    });
    it('should includes plural game tree', function() {
      return assert.deepEqual(parser.fastParse('(;)(;)'), [
        {
          _children: []
        }, {
          _children: []
        }
      ]);
    });
    return it('should includes plural game tree', function() {
      return assert.deepEqual(parser.fastParse('(;)\n(;)'), [
        {
          _children: []
        }, {
          _children: []
        }
      ]);
    });
  });
  describe('game tree', function() {
    it('should includes empty value of property ', function() {
      return assert.deepEqual(parser.fastParse('(;TB)'), [
        {
          _children: [],
          TB: null
        }
      ]);
    });
    it('should includes empty value of property ', function() {
      return assert.deepEqual(parser.fastParse('(;TB FF[4])'), [
        {
          _children: [],
          TB: null,
          FF: '4'
        }
      ]);
    });
    it('should includes a normal game tree', function() {
      return assert.deepEqual(parser.fastParse('(;FF[4])'), [
        {
          _children: [],
          FF: '4'
        }
      ]);
    });
    it('should includes a normal game tree', function() {
      return assert.deepEqual(parser.fastParse('(;FF[4]GM[1])'), [
        {
          _children: [],
          FF: '4',
          GM: '1'
        }
      ]);
    });
    it('should includes a normal game tree', function() {
      return assert.deepEqual(parser.fastParse('(;FF[4]GM[1]EV[])'), [
        {
          _children: [],
          FF: '4',
          GM: '1',
          EV: ''
        }
      ]);
    });
    it('should includes a normal game tree', function() {
      return assert.deepEqual(parser.fastParse('(;FF[4]GM[1]EV[ ])'), [
        {
          _children: [],
          FF: '4',
          GM: '1',
          EV: ' '
        }
      ]);
    });
    it('should treats a soft line break', function() {
      return assert.deepEqual(parser.fastParse('(;FF[4]GM[1]EV[a\\\nb])'), [
        {
          _children: [],
          FF: '4',
          GM: '1',
          EV: 'ab'
        }
      ]);
    });
    it('should treats an escaped ]', function() {
      return assert.deepEqual(parser.fastParse('(;FF[4]GM[1]EV[\\]])'), [
        {
          _children: [],
          FF: '4',
          GM: '1',
          EV: ']'
        }
      ]);
    });
    it('should treats an escaped ]', function() {
      return assert.deepEqual(parser.fastParse('(;FF[4]GM[1]EV[()])'), [
        {
          _children: [],
          FF: '4',
          GM: '1',
          EV: '()'
        }
      ]);
    });
    it('should  an escaped ]', function() {
      return assert.deepEqual(parser.fastParse('(;DT[2015-09-03];B[pd](;W[nq])(;W[aa]))'), [
        {
          _children: [
            {
              _children: [
                {
                  _children: [],
                  W: 'nq'
                }, {
                  _children: [],
                  W: 'aa'
                }
              ],
              B: 'pd'
            }
          ],
          DT: '2015-09-03'
        }
      ]);
    });
    it('long sequence with linebreak', function() {
      var a;
      a = parser.fastParse('(;A[][][][][][][][][][][][][][][][][][][][][][][][][][][][][\n])');
      return assert.equal(typeof a, 'object');
    });
    return it('wrong sequence with linebreak', function() {
      var a, e;
      a = (function() {
        var error;
        try {
          return parser.fastParse('(;A[][][][][][][][][][][][][][][][][][][][][][][][][][][][][\n]');
        } catch (error) {
          e = error;
          return null;
        }
      })();
      return assert.equal(a, null);
    });
  });
  describe('stringify', function() {
    it('should return string', function() {
      var sgf;
      sgf = '(;FF[4])';
      return assert.equal(parser.stringify(parser.fastParse(sgf)), sgf);
    });
    return it('should make FF prior', function() {
      var sgf;
      sgf = '(;GM[1]FF[4])';
      return assert.equal(parser.stringify(parser.fastParse(sgf)), '(;FF[4]GM[1])');
    });
  });
  describe('isSgf', function() {
    return it('should return true', function() {
      var sgf;
      sgf = '(;FF[4])';
      return assert.equal(parser.isSgf(sgf), true);
    });
  });
  return describe('nthMoveNode', function() {
    it('should return root', function() {
      var root;
      root = parser.fastParse('(;FF[4]SZ[19];B[aa])')[0];
      return assert.equal(parser.nthMoveNode(root, 0), root);
    });
    it('should return first move node', function() {
      var root;
      root = parser.fastParse('(;FF[4]SZ[19];B[aa])')[0];
      return assert.equal(parser.nthMoveNode(root, 1), root._children[0]);
    });
    return it('should return first move node', function() {
      var root;
      root = parser.fastParse('(;FF[4]SZ[19];B[aa];W[bb];B[cc];W[dd])')[0];
      return assert.equal(parser.nthMoveNode(root, Infinity).W, 'dd');
    });
  });
});
